#include <iostream>
#include <assert.h>
#include "nethogs.h"
#include "connection.h"

class ConnList
{
public:
	ConnList (Connection * m_val, ConnList * m_next)
	{
	    val = m_val; next = m_next;
	}
	Connection * val;
	ConnList * next;
};

ConnList * connections = NULL;

Connection::Connection (Packet * packet)
{
	if (DEBUG)
		assert (packet != NULL);
	connections = new ConnList (this, connections);
	packets = new PackList (packet, NULL);
	refpacket = packet->newPacket ();
	lastpacket = packet->time.tv_sec;
	if (DEBUG)
		cout << "New reference packet created at " << refpacket << endl;
}

Connection::~Connection ()
{
	if (DEBUG)
		cout << "Deleting connection" << endl;
	delete refpacket;
    	if (packets != NULL)
		delete packets;
}

bool timesmatch (timeval one, timeval other)
{
	return (one.tv_sec == other.tv_sec);
}

void Connection::add (Packet * packet)
{
	lastpacket = packet->time.tv_sec;
	if (packets == NULL)
	{
		//if (DEBUG)
		//	cout << "Adding new packet at " << packet << endl;
		packets = new PackList (packet, NULL);
		return;
	}

	if (timesmatch (packets->getVal()->time, packet->time))
	{
		packets->getVal()->len += packet->len;
		delete packet;
		return;
	}

	//if (DEBUG)
	//	cout << "Adding package to list at " << packet << endl;
	packets = new PackList (packet, packets);
}

Connection * findConnection (Packet * packet)
{
	ConnList * current = connections;
	while (current != NULL)
	{
		if (packet->match(current->val->refpacket))
			return current->val;

		current = current->next;
	}
	return NULL;
}

bool istooold (timeval time, timeval now)
{
    return (time.tv_sec + PERIOD <= now.tv_sec);
}

bpf_u_int32 Connection::sumanddel (timeval t)
{
    bpf_u_int32 sum=0;
    if (! packets)
	    return sum;

    PackList * curpack = packets;
    PackList * lastpack = NULL;
    while (curpack != NULL)
    {
	if (DEBUG)
		assert (curpack->getVal() != NULL);
	if (istooold(curpack->getVal()->time, t))
	{
	    if (curpack == packets)
	    {
		    packets = NULL;
		    delete (curpack);
    	    	    //cout << "packlist afterr: none." << endl;
	    }
	    else
	    {
		    if (DEBUG)
			    assert (lastpack != NULL);
		    lastpack->next = NULL;
		    delete (curpack);
    	    	    //cout << "packlist afterr: " << packets << "==> "; packets->print();
    	            //cout << endl;
	    }
	    return sum;
	}
	sum += curpack->getVal()->len;
	lastpack = curpack;
	curpack = curpack->next;
    }
    return sum;
}
