#include "nethogs.h"

#include <iostream>
#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <signal.h>
#include <string>
#include <string.h>

#include <ncurses.h>

extern "C" {
	#include <pcap.h>
}

#include "packet.h"
#include "connection.h"
#include "process.h"
#include "refresh.h"

bool needrefresh = false;
timeval curtime;
string * caption;

void process (u_char * args, const struct pcap_pkthdr * header, const u_char * m_packet)
{
	curtime = header->ts;
	Packet * packet = getPacket (header, m_packet);
	if (packet == NULL)
		return;

	Connection * connection = findConnection(packet);
	if (connection != NULL)
	{
		connection->add(packet);
		return;
	}
	connection = new Connection (packet);
	Process * process = getProcess(connection);
	//process->addConnection (connection);
	
	if (needrefresh)
	{
		do_refresh();
		needrefresh = false;
	}

	return;
}

void quit_cb (int i)
{
	procclean();
	clear();
	endwin();
	exit(0);
}

void main (int argc, char** argv)
{
#if DEBUG
#else
	initscr();
	raw();
	noecho();
	cbreak();
#endif
	char* dev;
	if (argc == 2)
	{
		dev = argv[1];
		cerr << "Read device " << dev << " from commandline\n";
	}
	else
	{
		dev = strdup("eth0");
		cerr << "defaulting to " << dev << endl;
	}
	caption = new string ("NetHogs ");
	caption->append(VERSION);
	caption->append(", running at ");
	caption->append(dev);

	if (NEEDROOT && (getuid() != 0))
	{
		clear();
		endwin();
		cerr << "You need to be root to run NetHogs!\n";
		exit(0);
	}

	char errbuf[PCAP_ERRBUF_SIZE];

	pcap_t * handle;
	handle = pcap_open_live(dev, BUFSIZ, 0, 1000, errbuf);

	signal (SIGALRM, &alarm_cb);
	signal (SIGINT, &quit_cb);
	alarm (REFRESHRATE);
	while (1)
	{
		pcap_dispatch (handle, -1, process, NULL);
		if (needrefresh)
		{
			do_refresh(); 
			needrefresh = false;
		}
	}
}

